package de.oltzen.awt;
import java.util.*;
import java.awt.*;

/** Layout in dem alle Component in einer Reihe sind.
 (c) 1998 by Thomas Oltzen
 */
@SuppressWarnings("serial")
public class HorizontalLayout implements java.awt.LayoutManager, java.io.Serializable
{

	private Hashtable htable;

	private int hgap;
    private int vgap;

	private int height = 0;


    /** Erstellt ein Layout in dem alle Component in einer Reihe sind. 
     * 
     */
    public HorizontalLayout()
	{
		htable = new Hashtable();
		hgap = 0;
		vgap = 0;
    }

	/**
	 * @param h
	 */
	public void setHeight( int h)
	{
		height = h;
	}

	/** Get the height of all component.
	 * @return
	 */
	public int getHeight()
	{
		return height;
	}

    /**
     * Gets the horizontal gap between components.
     * @return       the horizontal gap between components.
     */
    public int getHgap()
	{
		return hgap;
    }

    /**
     * Sets the horizontal gap between components to the specified value.
     * @param        hgap   the horizontal gap between components.
     */
    public void setHgap(int hgap)
	{
		this.hgap = hgap;
    }

    /**
     * Gets the vertical gap between components.
     * @return       the vertical gap between components.
     */
    public int getVgap()
	{
		return vgap;
    }


    /** Set the vertical gap.
     * @param vgap
     */
    public void setVgap(int vgap)
	{
		this.vgap = vgap;
    }

	/** Set the relative width of a component. 
	 * @param comp
	 * @param w
	 */
	public void setRelWidth(Component comp, int w)
	{
		htable.put(comp, new RelativeWidth(w));
	}

	public void setFixWidth(Component comp, int w)
	{
		htable.put(comp , new FixWidth(w));
	}


    public void addLayoutComponent(String name, Component comp)
	{
		htable.put(comp, new RelativeWidth(10));
    }



    public void removeLayoutComponent(Component comp)
	{
		htable.put(comp, null);
    }

    /**

     */
    public Dimension preferredLayoutSize(Container parent)
	{
		return new Dimension(10, height);
    }

    /**

     */
    public Dimension minimumLayoutSize(Container parent)
	{
		return new Dimension(1,1);
    }


    public void layoutContainer(Container parent)
	{
		int ncomponents = parent.getComponentCount();
		int fixlength;
		int relsum = 0;
		//int relwidth;
		//Compute the fixlength and relative sum.
		fixlength = hgap;
		for (int i = 0 ; i < ncomponents ; i++)
		{
			HWidth width = (HWidth) (htable.get(parent.getComponent(i)));
			if ((width != null) && (width.isFix()))
			{
				fixlength = fixlength + width.getValue();
			}
			else if (width != null)
			{
				relsum = relsum + width.getValue();
			}
			fixlength = fixlength + hgap;
		}
		// Set the bounds.
		Insets insets = parent.getInsets();
		int pw = parent.getSize().width ;// - (insets.left + insets.right);
		//System.out.println("parent.width="+pw+ "  relsum="+relsum);
		int ph = parent.getSize().height - (insets.top + insets.bottom);
		int comp_height;
		if (height<=0)
		{
			comp_height = ph-2*vgap;
		}
		else
		{
			comp_height = height;
		}
		int xpos = vgap;
		for (int i = 0 ; i < ncomponents ; i++)
		{
			HWidth width = (HWidth) (htable.get(parent.getComponent(i)));
			if ((width != null) && (width.isFix()))
			{
				parent.getComponent(i).setBounds(xpos, hgap, width.getValue(), comp_height);
				xpos = xpos + width.getValue();
			}
			else if (width != null)
			{
				int comp_width = (pw-2*hgap-fixlength)*width.getValue()/relsum;
				parent.getComponent(i).setBounds(xpos, hgap, comp_width, comp_height);
				xpos = xpos + comp_width;
			}
			xpos = xpos + vgap;
		}

    }


    public String toString()
	{
		return getClass().getName() ;
    }
}

