/* (c) 2004 Thomas Oltzen
 * Created on 12.01.2004
 *
 */
package de.oltzen.awt;

import java.awt.event.ActionEvent;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JComponent;

/**
 * @author Thomas Oltzen
 * Create : 16:51:35
 *
 */
@SuppressWarnings("serial")
public class MethodActionPerformer
implements java.awt.event.ActionListener, Serializable
{
	private Hashtable<Object, String> theActionMethod;
	private Hashtable<String, Method> theMName2Method;
    private Hashtable<String, Object> theMName2Object;	
	private boolean theHasMessageFlag;
	
	private Vector<Object> theMethodObjectVector;
    
    private HashSet<String> theProgressMNameSet = new HashSet<String>();
    private ProgressPerformer theProgressPerformer;
	
	/** Alle Methoden werden auf das mobject ausgefhrt. Das mobject ist das Hauptobject.
	 * Wenn es ein ICallBackReceiver ist, wird die dazugehrige Methode aufgerufen. 
	 * Dies ist zum Beispiel bei einer "Zurck" Funktionalitt wichtig. */ 
	public MethodActionPerformer(Object mobject)
	{
		theActionMethod = new Hashtable<Object, String>();
		theMName2Method = new Hashtable<String, Method>();
		theMName2Object = new Hashtable<String, Object>();
		theMethodObjectVector = new Vector<Object>();
		theMethodObjectVector.add(mobject);
	}

    /** Auf dem 'obj' wird der Actionlisterner hinzugefgt. Der Methodennamen bestimmte die Methode.
     * Es muss mindestends ein Object in der Liste theMethodObjectVector geben, welches eine Methode mit diesem Namen enthlt.
     * @param obj
     * @param methodname
     */
	public void watch(Object obj, String methodname) {
        for (int i = 0 ; i< theMethodObjectVector.size() ;i++){
            Method[] ma = theMethodObjectVector.get(i).getClass().getMethods();
            for (int j = 0 ;j < ma.length ; j++){
                if (	ma[j].getName().equals(methodname) &&
                        (ma[j].getParameterTypes().length ==0 ||
                                ma[j].getParameterTypes().length ==1 &&
                                ma[j].getParameterTypes()[0]==ProgressAdvance.class)  &&
                        Modifier.isPublic(ma[j].getModifiers()))
                {
                    theActionMethod.put(obj, methodname);
                    theMName2Method.put(methodname, ma[j]);
                    theMName2Object.put(methodname, theMethodObjectVector.get(i));
                    if (ma[j].getParameterTypes().length ==1){
                        addProgressMethodname(methodname);
                    }
                    theHasMessageFlag = true;
                    return;
                }
            }
        }
        theHasMessageFlag = false;
        System.err.println("Warning! The object " + theMethodObjectVector.get(0).getClass() + " has not the method "
                + methodname + ".");
 
    }

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		try {
		    String methodname = theActionMethod.get(arg0.getSource());
			if (methodname!=null){
                if (theProgressMNameSet.contains(methodname)){
                    theProgressPerformer.perform(methodname);
                } else {
    			    Object goal = theMName2Object.get(methodname);
    				Method m = theMName2Method.get(methodname);
    				m.invoke(goal, new Object[0]);
                }
			}
		} catch (InvocationTargetException exc) {
			Throwable texc = exc.getTargetException();
            ThrowInformer.inform(texc);
			texc.printStackTrace();
			if (arg0.getSource() instanceof IActionOutput){
				((IActionOutput)arg0.getSource()).printException(texc);
			}
			
		} catch (Exception exc)
		{
            ThrowInformer.inform(exc);
			if (arg0.getSource() instanceof IActionOutput){
				((IActionOutput)arg0.getSource()).printException(exc);
			}
		}
		
	}

    /**
     * Set all user interface item to a visisble status.
     */
    public void setVisible (String methodname, boolean visible)
    {
        Object[] obj_a = theActionMethod.keySet().toArray();
        Vector res = new Vector();
        for (int i = 0 ; i < obj_a.length ; i++){
            if (theActionMethod.get(obj_a[i]).equals(methodname) && obj_a[i] instanceof JComponent){
                ((JComponent) obj_a[i]).setVisible(visible);
            }
        }
    }
    
    /**
     * @return Returns the hasMessageFlag.
     */
    public boolean hasMessageFlag() {
        return theHasMessageFlag;
    }

    /**
     * @param obj
     */
    public void addMethodActionObject(Object mobject) {
        theMethodObjectVector.add(mobject);
    }
    
    /**
     * @param obj
     */
    private void addProgressMethodname(String methodname) {
        theProgressMNameSet.add(methodname);
    }

    
    public void setPerformer(ProgressPerformer map)
    {
        theProgressPerformer = map;
    }

    public void actionProgress(ProgressAdvance progressadvance, Object obj)
    {
        try {
            String methodname = obj.toString();
            if (methodname!=null){
                    Object goal = theMName2Object.get(methodname);
                    Method m = theMName2Method.get(methodname);
                    Object[] para = new Object[1];
                    para[0] = progressadvance;
                    m.invoke(goal, para);
            }
        } catch (InvocationTargetException exc) {
            Throwable texc = exc.getTargetException();
            ThrowInformer.inform(texc);
            texc.printStackTrace();
            
        } catch (Exception exc)
        {
            ThrowInformer.inform(exc);
        }
    }

 }
