/* (c) by Thomas Oltzen
 * Created on 22.02.2005
 *
 */
package de.oltzen.awt;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.JRadioButtonMenuItem;

/**
 * @author Thomas Oltzen
 *
 */
@SuppressWarnings("serial")
public class TomFrame extends JFrame 
implements ActionListener
{

    private TableLayout theLayout;
    private MethodActionPerformer theMAPerfomer;

    protected JMenuBar theJMenuBar;
    protected JMenu theCurrentJMenu;

    public TomFrame(int row, int col)
    {
        theLayout = new TableLayout(row,col);
        setLayout(theLayout);
        theMAPerfomer = new MethodActionPerformer(this);
        theMAPerfomer.setPerformer(new ProgressPerformer(this, new MyProgressFunction()));
        theJMenuBar = new JMenuBar();
       
    }

    /** Insert a child component into the component. */
    public void insertComp ( Component comp, int row, int col)
    {
        insertComp(comp,row,col,1,1);
    }

    /** Insert a child component into the component. Additional
    * the information of the joined fields.
    */
    public void insertComp
    (
        Component comp,
        int row,
        int col,
        int rowjoin,
        int coljoin
    )
    {
        theLayout.addComponent(comp,row,col,rowjoin,coljoin);
        add(comp);
    }

    /** Get the layout manager. */
    public TableLayout getTableLayout()
    {
        return theLayout;
    }

    public void addMethodActionObject(Object obj){
        theMAPerfomer.addMethodActionObject(obj);
    }
    
    public void createMenu(String title)
    {
        theCurrentJMenu = new JMenu(title);
        //theCurrentJMenu.setBackground(new Color(216,213,186));
        theJMenuBar.add(theCurrentJMenu);
    }

    public JMenuItem createMenuItem(String title)
    {
        JMenuItem mi=new JMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(this);
        return mi;
    }
    
    public void createMenuSeperator()
    {
        theCurrentJMenu.addSeparator();
    }


    /** Create a new menu item under the last created menu.
     * The inherented class must ontain a menthodname without a parameter.
     * If a user select this menu item, then call it this method.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JMenuItem createMenuItem(String title, String methodname)
    {
        JMenuItem mi=new JMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(mi, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            mi.setEnabled(false);
        }
        return mi;
    }

    /** Create a radio button.
     *
     * @param title
     * @return
     */
    public JRadioButtonMenuItem createRadioButtonMenuItem(String title)
    {
        JRadioButtonMenuItem mi=new JRadioButtonMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(this);
        return mi;
    }

    /** Create a radio button.
     * The inherented class must ontain a menthodname without a parameter.
     * If a user select this menu item, then call it this method.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JRadioButtonMenuItem createRadioButtonMenuItem(String title, String methodname)
    {
        JRadioButtonMenuItem mi=new JRadioButtonMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(mi, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            mi.setEnabled(false);
        }
        return mi;
    }

    public JCheckBoxMenuItem createCheckBoxMenuMenuItem(String title)
    {
        JCheckBoxMenuItem mi=new JCheckBoxMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(this);
        return mi;
    }

    /** Create a new check box menu item under the last created menu.
     * The inherented class must ontain a menthodname without a parameter.
     * If a user select this menu item, then call it this method.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JCheckBoxMenuItem createCheckBoxMenuMenuItem(String title, String methodname)
    {
        JCheckBoxMenuItem mi=new JCheckBoxMenuItem(title);
        theCurrentJMenu.add(mi);
        mi.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(mi, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            mi.setEnabled(false);
        }
        return mi;
    }

    public JMenuBar getJMenuBar()
    {
        return theJMenuBar;
    }

    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent arg0) {
        // TODO: Diese Methode soll Stck fr Stck elemeniert werden.
    }


    /** Create a button. The button will be add to the frame.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JButton createButton(String title, String methodname)
    {
        JButton button=new JButton(title);
        button.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(button, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            button.setEnabled(false);
        }
        return button;
    }

    /** Create a button. The button will be add to the frame.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JButton createButton(Icon icon, String methodname)
    {
        JButton button=new JButton(icon);
        button.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(button, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            button.setEnabled(false);
        }
        return button;
    }

    /** Create a radio button. The button will be add to the frame.
     *
     * @param title
     * @param methodname
     * @return
     */
    public JRadioButton createRadioButton(String title, String methodname)
    {
        JRadioButton button=new JRadioButton(title);
        button.addActionListener(theMAPerfomer);
        theMAPerfomer.watch(button, methodname);
        if (!theMAPerfomer.hasMessageFlag()){
            button.setEnabled(false);
        }
        return button;
    }
    
    class MyProgressFunction implements ProgressFunction
    {

        public void performProgress(ProgressAdvance progressadvance, Object obj)
        {
            theMAPerfomer.actionProgress(progressadvance, obj);
            
        }
        
    }
    
}
