//$Id: FwTree.java,v 1.1 2009/09/02 20:08:21 Administrator Exp $
package de.oltzen.fw.tree;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

/** Sorry, the comments are in german. I hope I have time for a good
* documentation.<br>
* Enthlt einige Verwaltungsroutinen zur Manipulation des Baumes.
* <b>&copy; 1999 by Thomas Oltzen, under the terms of the LGPL</b>
* @version 1.0
* @author Thomas Oltzen
*/

public class FwTree
{

    private JTree theTree;

    public FwTree (FwTreeFrame frame, FwComponent root_element)
    {
        DefaultMutableTreeNode root_node =
            new DefaultMutableTreeNode(root_element);
        theTree = new JTree(root_node);
        theTree.addTreeSelectionListener (new FwTreeSelectionListener(frame));
        theTree.setCellRenderer(new FwTreeCellRenderer());
        theTree.setMinimumSize(new Dimension(30, 30));
        theTree.setLargeModel(true);
    }

    /** Liefert die Tree-Component. */
    public JTree getJTree()
    {
        return theTree;
    }

    public void updateTree()
    {
        ((DefaultTreeModel)theTree.getModel()).reload();
        //theTree.treeDidChange();
    }

    /** Ersetzt eine Komponente im Baum durch eine andere Komponente.
    */
    public void replace (FwComponent old_tele, FwComponent new_tele)
    {
        DefaultMutableTreeNode node = getTreeNode(old_tele);
        node.setUserObject(new_tele);
    }

    /** Lscht eine Komponente. Diese darf keine Kinder im Baum haben.
    * Die Wurzel kann nicht gelscht werden.
    */
    public void remove (FwComponent tele)
    {
        getTreeNode(tele).removeFromParent();
    }

	/** Lscht alle Children von einer Branch. */
	public void removeAllChildren(FwComponent tele)
	{
		getTreeNode(tele).removeAllChildren();
	}

    /* Setzt die Komponente als Root. Diese Methode mu vor den Methoden
    * <code>replace, remove, insertBehind, insertChildAsFirst</code> und
    * <code>insertChildAsLast</code> aufgerufen werden. Der ganze Baum wird
    * durch diese Methode gelscht und mit der neuen Wurzel versehen.
    */
    // Erst in JDK1.2 gut und einfach zu programmieren.
    //public void insertAsRoot (GComponent comp)
    //{
    //
    //}

    /** Fgt die Komponente <code>comp_behind</code> hinter einer Komponente <code>comp_front</code> ein.
    * Die eingefgte Komponente erscheint in der gleichen Ebene wie die andere Komponente
    */
    public void insertBehind (FwComponent tele_front, FwComponent tele_behind)
    {
        DefaultMutableTreeNode front_node = getTreeNode(tele_front);
        DefaultMutableTreeNode parent_node = (DefaultMutableTreeNode) front_node.getParent();
        DefaultMutableTreeNode comp_behind_node =
            new DefaultMutableTreeNode(tele_behind);
        int front_index = parent_node.getIndex(front_node);
        parent_node.insert(comp_behind_node, front_index+1);
		((DefaultTreeModel)theTree.getModel()).reload(parent_node);
    }

    /** Fgt die Komponente <code>comp_child</code> als Blatt zur Komponente <code>comp_child</code> ein.
    * Sie wird in der Child-Ebene am Anfang eingefgt.
    */
    public void insertChildAsFirst (FwComponent tele_parent, FwComponent tele_child)
    {
        DefaultMutableTreeNode parent_node = getTreeNode(tele_parent);
        DefaultMutableTreeNode child_node =
            new DefaultMutableTreeNode(tele_child);
        parent_node.insert(child_node, 0);
		((DefaultTreeModel)theTree.getModel()).reload(parent_node);
    }

    /** Fgt die Komponente <code>comp_child</code> als Blatt zur Komponente <code>comp_child</code> ein.
    * Sie wird in der Child-Ebene am Ende eingefgt.
    */
    public void insertChildAsLast (FwComponent tele_parent, FwComponent tele_child)
    {
        DefaultMutableTreeNode parent_node = getTreeNode(tele_parent);
        DefaultMutableTreeNode child_node =
            new DefaultMutableTreeNode(tele_child);
        //System.out.println("parent="+parent_node);
        //System.out.println("child="+child_node);

        parent_node.add(child_node);
		((DefaultTreeModel)theTree.getModel()).reload(parent_node);
    }

    /** Liefert die Wurzelkomponente.*/
    public FwComponent getRoot()
    {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)theTree.getModel().getRoot();
        FwComponent tele = (FwComponent) node.getUserObject();
        return tele;
    }

    /**  Liefert die Anzahl der Child-Komponente die zu einer Komponente existierten. */
    public int getNumberOfChild (FwComponent tele_parent)
    {
        TreeNode node = getTreeNode(tele_parent);
        return node.getChildCount();
    }

    /** Liefert eine Komponente vom Child-Zweig bezglich der Parent-Komponente. */
    public FwComponent getChild (FwComponent tele_parent, int position)
    {
        DefaultMutableTreeNode node =
            (DefaultMutableTreeNode) getTreeNode(tele_parent).getChildAt(position);
        return (FwComponent) node.getUserObject();
    }

    /** Markiert die Komponente als selektiert. Z.Z nicht implementiert.*/
    public void setAsSelected (FwComponent tele)
    {
    }

    /**  Sucht den Baumknoten fr eine Komponente. */
    private DefaultMutableTreeNode getTreeNode(FwComponent tele)
    {
        return getTreeNodeSub((DefaultMutableTreeNode)theTree.getModel().getRoot(), tele);
    }

    /** Unterroutine zum suchen der Komponente im Baumknoten. */
    private DefaultMutableTreeNode getTreeNodeSub
		(DefaultMutableTreeNode tree_node, FwComponent tele)
    {
        if (tree_node == null)
        {
            return null;
        }
        if ( ((FwComponent) tree_node.getUserObject()) == tele)
        {
            return tree_node;
        }
        for (int i = 0 ; i < tree_node.getChildCount() ; i ++)
        {
            DefaultMutableTreeNode child  =
                  (DefaultMutableTreeNode) tree_node.getChildAt(i);
            if (child != null)
            {
                DefaultMutableTreeNode child_result = getTreeNodeSub(child, tele);
                if (child_result != null)
                {
                    return child_result;
                }
            }
        };
        return null;
    }

}
