//$Id: FwTreeFrame.java,v 1.1 2009/09/02 20:08:21 Administrator Exp $
package de.oltzen.fw.tree;

import java.lang.reflect.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import de.oltzen.awt.ThrowInformer;

/** Sorry, the comments are in german. I hope I have time for a good
* documentation.<br>
* <b>&copy; 1999 by Thomas Oltzen, under the terms of the LGPL</b>
* @version 1.0
* @author Thomas Oltzen
*/
public class FwTreeFrame extends JFrame implements WindowListener
{

	/** Assoziation zwischen dem MenuItem-Symbol (ein String) und
	 * dem MenuItem.
	 */
	Hashtable theSymMIAktAssoc;

	private FwComponent theInvisibleRoot;

	private FwTree theTree;

	/** Contain the actual component on the right side of the frame. */
	private FwComponent theViewComponent;

	public JSplitPane theSplitPane;

	private Menu theActionMenu;

	public FwTreeFrame ()
	{
		theSymMIAktAssoc = new Hashtable();
		theInvisibleRoot = new FwRootComponent();

		MenuBar mb = new MenuBar();
		mb.add(new Menu("File"));
		setMenuBar(mb);
		initStdMenuBar();

		setSize(700,500);

		theSplitPane = new JSplitPane();
		theSplitPane.setContinuousLayout(true);
		
		getContentPane().add(theSplitPane);
		theTree = new FwTree( this, theInvisibleRoot );
		theTree.getJTree().expandRow(0);
		theTree.getJTree().setRootVisible(false);
		theSplitPane.setLeftComponent(new JScrollPane(theTree.getJTree()));
		addWindowListener(this);
	}

	public void addActionMenu()
	{
		MenuBar mb = getMenuBar();
		theActionMenu = new Menu("Action");
		mb.add(theActionMenu);
	}

	public void setViewable (FwComponent viewable)
	{
		if (theViewComponent != null)
		{
			theViewComponent.disappearFromFrame();
		}
		removeAllMenuItems();
		viewable.viewOnFrameAction(this);
		theSplitPane.setRightComponent(new JScrollPane(viewable.getFrameComponent()));
		theViewComponent = viewable;
	}

	public void viewOnRight (Component viewable)
	{
		theSplitPane.setRightComponent(new JScrollPane(viewable));
	}

	public void windowClosing(WindowEvent e)
	{
		setVisible(false);
		//displayMessage("Window closing", e);
		exit();
	}

	public void windowClosed(WindowEvent e)
	{
		//displayMessage("Window closed", e);
	}

	public void windowOpened(WindowEvent e)
	{
		//displayMessage("Window opened", e);
	}

	public void windowIconified(WindowEvent e)
	{
		//displayMessage("Window iconified", e);
	}

	public void windowDeiconified(WindowEvent e)
	{
		//displayMessage("Window deiconified", e);
	}

	public void windowActivated(WindowEvent e)
	{
		//displayMessage("Window activated", e);
	}

	public void windowDeactivated(WindowEvent e)
	{
		//displayMessage("Window deactivated", e);
	}

	private void displayMessage(String prefix, WindowEvent e)
	{
		System.out.println(prefix
					   + ": "
					   + e.getWindow());
	}

	/** Erstellt eine Menubar fr das Frame. */
	public void initStdMenuBar ()
	{
	}

	/** Erzeugt ein neues MenuItem. Alle Menitems, die mit 'outside_action'
	* gekennzeichnet wurden, kann jede Komponente eine Aktion anhngen.
	* @param name Symbolic name. ber diesen wird das Menitem manipuliert.
	* @param titel Anzeigetitel im Men.
	* @param outside_action A flag that definite that a <code>FwComponent</code> be able attach an action.
	*/

	public void addFwMenuItem(String menutitle,
	                          String symbolname,
	                          String itemtitle)
	{
		addMenuItem(menutitle, symbolname, itemtitle, true, true);
	}

	public void addFwMenuItem(String menutitle,
	                          String symbolname,
	                          String itemtitle,
	                          Object menuobj,
	                          String methodname)
	{
		addMenuItem(menutitle, symbolname, itemtitle, true, true);
		setAction(symbolname, menuobj, methodname);
	}

	public void addFwMenuItemSolid(String menutitle,
	                          	   String symbolname,
	                               String itemtitle,
	                               Object menuobj,
	                               String methodname)
	{
		addMenuItem(menutitle, symbolname, itemtitle, false, false);
		setAction(symbolname, menuobj, methodname);
	}

	private void addMenuItem(String menutitle,
	                         String symbolname,
	                         String itemtitle,
	                         boolean removable_action,
	                         boolean removable_item)
	{
		MenuBar mb = getMenuBar();
		Menu menu = null;
		for (int i = 0 ; i < mb.getMenuCount() ; i++)
		{
			if (mb.getMenu(i).getLabel().equals(menutitle))
			{
				menu = mb.getMenu(i);
				i = mb.getMenuCount();
			}
		}
		if (menu == null)
		{
			menu = new Menu(menutitle);
			mb.add(menu);
		}
		MenuItem mi = new MenuItem(itemtitle);
		menu.add(mi);
		mi.setEnabled(false);
		FwMenuAction makt = new FwMenuAction();
		mi.addActionListener(new FwMenuActionListener(makt,this));
		makt.theName = symbolname;
		makt.theMenu = menu;
		makt.theMenuItem = mi;
		makt.theRemovableItem = removable_item;
		makt.theRemovableAction = removable_action;
		theSymMIAktAssoc.put(symbolname, makt);
	}

	/** This is a special menthod for add a menu item. A menu item
	* that create outside of the framework and it add to the menu bar
	* is for the framework undefined. But this item are depend for
	* some <code>FwComponent</code>.
	*/
	public void includeMenuItem(MenuItem item,
								Menu menu,
	                            String symbolname,
	                            boolean outside_action)
	{
		FwMenuAction makt = new FwMenuAction();
		item.addActionListener(new FwMenuActionListener(makt, this));
		makt.theName = symbolname;
		makt.theMenu = menu;
		makt.theMenuItem = item;
		makt.theRemovableItem = false;
		makt.theRemovableAction = outside_action;
		theSymMIAktAssoc.put(symbolname, makt);
	}

	/** Setzt alle Menitems auf disable. Es sind aber die ausgenommen, die
	* vom Frame direkt genutzt werden. Also mit <code>outside_aktion=false</code>
	* in der Methode <code>createMenuItem</code> erstellt wurden.
	*/
	private void removeAllMenuItems ()
	{
		Enumeration ele = theSymMIAktAssoc.elements();
		for (; ele.hasMoreElements() ;)
		{
			FwMenuAction makt = (FwMenuAction) ele.nextElement();
			if (makt.theRemovableItem == true)
			{
				makt.theMenu.remove(makt.theMenuItem);
				if (makt.theMenu.getItemCount() == 0)
				{
					getMenuBar().remove(makt.theMenu);
				}
			}
			else
			{
				if (makt.theRemovableAction == true)
				{
					makt.theMenuItem.setEnabled(false);
				}
			}
		}
		if ( theActionMenu != null)
			theActionMenu.removeAll();
	}

	/** Setzt ein Menitem auf disable. */
	public void disableMenuItem(String symbolname)
	{
		FwMenuAction makt = (FwMenuAction)theSymMIAktAssoc.get(symbolname);
		if (makt != null)
		{
			makt.theMenuItem.setEnabled(false);
		}

	}

	/** Setzt eine Aktion fr ein MenuItem. Dabei wird dieser auch auf 'enable'
	* gesetzt.
	*/
	public void setAction(String symbolname, Object menuobj, String methodname)
	{
		FwMenuAction maction = (FwMenuAction) theSymMIAktAssoc.get(symbolname);
		try
		{
			maction.theMethod =
				menuobj.getClass().getMethod(methodname, new Class[0]);
		}
		catch (Exception exc)
		{
            ThrowInformer.inform(exc);
		}
		maction.theMenuObject = menuobj;
		maction.theMenuItem.setEnabled(true);
	}

	/** Erweitert eine Aktion zum Menu "Aktion" erweitert.  */
	public void addAction(String title, Object menuobj, String methodname)
	{
		MenuItem mi = new MenuItem(title);
		mi.setEnabled(true);
		FwMenuAction makt = new FwMenuAction();
		mi.addActionListener(new FwMenuActionListener(makt, this));
		makt.theName = title;
		makt.theMenuItem = mi;
		try
		{
			makt.theMethod =
				menuobj.getClass().getMethod(methodname, new Class[0]);
		}
		catch (Exception exc)
		{
            ThrowInformer.inform(exc);
		}
		makt.theMenuObject = menuobj;
		theActionMenu.add(mi);
	}

	public void exit ()
	{
		//System.out.println("Programm wird jetzt beendet.");
		System.exit(0);
	}

	/** Get the tree of the frame. All external manipulation from the tree
	* are over the <code>FwTree</code>. Would you like append a child at a
	* tree element, then perform this statement:<br>
	* <pre>
	*      FwComponent parent = .... // must be a element from the tree.
	*      FwComponent child = new MyImplOfFwComponent(...);
	*      frame.getFwTree().insertChildAsLast(parent, child);
	* </pre>
	*/
	public FwTree getFwTree () { return theTree; };

	public FwComponent getInvisibleRoot () {return theInvisibleRoot;}

	public void clearTreeSelection ()
	{
		theTree.getJTree().clearSelection();
	}
}

class FwMenuAction
{
	String 		theName;
	Object  	theMenuObject;
	Method 		theMethod;
	Menu        theMenu;
	MenuItem 	theMenuItem;
	boolean     theRemovableItem;
	boolean 	theRemovableAction;
}

class FwMenuActionListener implements java.awt.event.ActionListener
{
		FwMenuAction theMenuAction;
		FwTreeFrame  theFrame;

		FwMenuActionListener (FwMenuAction mak, FwTreeFrame frame)
		{
			theMenuAction = mak;
			theFrame = frame;
		}

		public void actionPerformed(java.awt.event.ActionEvent e)
		{
			try
			{
				theFrame.clearTreeSelection();
	   			theMenuAction.theMethod.invoke(theMenuAction.theMenuObject, new Object[0]);
		    }
			catch (Exception exc)
			{
                ThrowInformer.inform(exc);
			}
		}
}

class FwRootComponent extends JComponent implements FwComponent
{
    public void viewOnFrameAction (FwTreeFrame frame){}

	public void disappearFromFrame () {}

	public JComponent getFrameComponent ()
	{
		return this;
	}

	public String getTreeBranchName ()
	{
		return "Root";
	}

}