/* (c) by Thomas Oltzen
 * Created on 27.08.2011
 *
 */
package de.oltzen.pianoteacher;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.TreeSet;

public class MidiConverter {
    
    MThd theMThd;
    
    ArrayList <MTrk> theTackList = new ArrayList <MTrk>();
    
    int theChanalFlag;
    
    TreeSet <Integer> theTimestamp = new TreeSet <Integer>();
    Hashtable <Integer, ArrayList<MTEvent>> theToneContent = new Hashtable <Integer, ArrayList<MTEvent>>();
    
    
    public static void main (String[] args) throws Exception{
        new MidiConverter().convert(new FileInputStream(args[0]), new FileOutputStream(args[0]+".txt"));
    }

    public void convert(InputStream is, OutputStream os) throws Exception{
        DataInputStream dis = new DataInputStream(is);
        String head = "" + (char) dis.read()+ (char) dis.read()+(char) dis.read()+ (char) dis.read();
        if (!head.equals("MThd")) throw new Exception("Wrong header. header="+head); 
        MThd mthd = new MThd();
        mthd.parser(dis);
        theMThd = mthd;
        for (int i= 0; i< mthd.theTrackCnt ; i++){
            MTrk track = new MTrk();
            String track_head = "" + (char) dis.read()+ (char) dis.read()+(char) dis.read()+ (char) dis.read();
            if (!track_head.equals("MTrk")) throw new Exception("Wrong track header. track_head="+track_head); 
            track.parser(dis, mthd.theDivisor);
            theTackList.add(track);
        }
        analyseTones();
        createMCCode(os);
    }
    
    private void createMCCode(OutputStream os) throws IOException
    {
        DataOutputStream dos = new DataOutputStream(os);
        dos.writeByte(0x3E); // Kanalbefehl
        dos.writeShort(theChanalFlag);
//        dos.writeByte(0x3d); // Benutzereingabe warten
        int pretime = 0;
        int gestime = 0;
        for (int time : theTimestamp){
            if (time!= pretime){
                int delta = (time-pretime)*666/theMThd.theDivisor;
                gestime += delta;
//                if (delta <256){
//                    dos.writeByte(0x31);
//                    dos.writeByte(delta);
//                } else if (delta <0x10000){
//                    dos.writeByte(0x32);
//                    dos.writeShort(delta);
//                } else {
                    dos.writeByte(0x34);
                    dos.writeInt(delta);
//                }
            }
            pretime = time;
            ArrayList<MTEvent> event_list = theToneContent.get(time);
            for (int channel = 0; channel < 15 ; channel++){
                createToneSeq(dos, channel, event_list, 0x80, 0x10);
                createToneSeq(dos, channel, event_list, 0x90, 0x20);
            }
        }
        
        System.out.println("total music time="+gestime);
        dos.writeByte(0x3B); // Music End Event
        dos.flush();
        
    }

    private void createToneSeq(DataOutputStream dos, int channel, ArrayList<MTEvent> eventList, int command, int ti_command) throws IOException
    {
        ArrayList<Integer> tones = new ArrayList<Integer>(10);
        for (MTEvent event : eventList){
            if (event.theChannel == channel && event.theCommand==command){
                tones.add(event.theTone);
            }
        }
        if (tones.size()>0){
            dos.writeByte(ti_command | channel);
            for (int tone : tones){
                dos.writeByte(tone);
            }
            dos.writeByte(0);
        }
        
    }

    private void analyseTones()
    {
        int gestime=0;
        for (int i= 0; i< theMThd.theTrackCnt ; i++){
            MTrk track = theTackList.get(i);
            int time = 0; 
            for (MTEvent e: track.theEventList){
                time += e.theDeltaTime;
                if (e.theCommand == 0x80 || e.theCommand == 0x90){
                 //   if (i==2) System.out.println("time = "+time+" note="+Integer.toHexString(e.theTone)+" c="+Integer.toHexString(e.theChanal)+" cmd:"+Integer.toHexString(e.theCommand));
                    theChanalFlag |= (1<<e.theChannel);
                    theTimestamp.add(time);
                    ArrayList<MTEvent> al = theToneContent.get(time);
                    if (al == null){
                        al = new ArrayList<MTEvent>();
                        theToneContent.put(time, al);
                    }
                    al.add(e);
                }
            }
            gestime+=time;
            System.out.println("time="+time/256.0);
        }
        System.out.println("chanals="+Integer.toHexString(theChanalFlag));
        System.out.println("gestime="+gestime/256.0);
    }

    public class MThd {
        int theLength;
        
        int theFormat;
        
        int theTrackCnt;
        
        /** Multiply by 4 is the second or full tone. */
       int theDivisor;
        
        public void parser(DataInputStream dis) throws Exception{
            theLength = dis.readInt();
            System.out.println("len="+Integer.toHexString(theLength));
            byte [] buffer = new byte[theLength];
            dis.read(buffer);
            DataInputStream bdis = new DataInputStream(new ByteArrayInputStream(buffer));
            theFormat = bdis.readShort();
            if (theFormat < 0 || theFormat >1) throw new Exception("Format error. format="+theFormat);
            theTrackCnt = bdis.readShort();
            theDivisor = bdis.readShort();
            System.out.println("div:"+theDivisor);
            if (theDivisor<0) throw new Exception("Divisor error. divisor="+theDivisor);
            bdis.close();
            
        }
    }
    
    public class MTrk {

        int theLength;
        
        ArrayList <MTEvent> theEventList;
        
//        int theToneCnt=0;
        
        public void parser(DataInputStream dis, int divisor) throws Exception{
            int divsel = divisor/16;
           // divsel =0;
            HashSet<Integer> double_note = new HashSet<Integer>();
            theLength = dis.readInt();
            System.out.println("len="+Integer.toHexString(theLength));
            byte [] buffer = new byte[theLength];
            dis.read(buffer);
            DataInputStream bdis = new DataInputStream(new ByteArrayInputStream(buffer));
            int value = parseFlexNumber(bdis);
            theEventList = new ArrayList <MTEvent>();
            int pre_command = 0;
            int val_rest=0;
            int pre_value;
            int ges_midi_time=0;
            while (value >= 0 ){
                ges_midi_time+= value;
                MTEvent e = new MTEvent();
                pre_value = value;
                if (value+val_rest<divsel){
                    val_rest += value;
                    value = 0;
                } else {
                    value += val_rest;
                    val_rest = 0;
                    double_note.clear();
                }
                e.theDeltaTime = value;
                if (value >0) System.out.print(""+value+":");
                parseEvent(e, bdis, pre_command);
                pre_command = e.theCommand+e.theChannel;
 //               if (e.theChanal!=1) theToneCnt = 0100;
                if (e.theCommand==0x90 && e.theStrength==0){
                    e.theCommand = 0x80; 
                }
                if (e.theCommand==0x90){
                   
                    if (double_note.contains(e.theTone)){
                        System.err.println("Double Tone!");
                        val_rest -= value;
                        e.theDeltaTime = pre_value;
                        System.out.print(""+pre_value+":");
                    }
                    System.out.println("On Tone:"+e.theTone);
                    double_note.add(e.theTone);
                }
                if (e.theCommand==0x80){
                    System.out.println("Off Tone:"+e.theTone);
                    if (double_note.contains(e.theTone)){
                        System.err.println("Double Tone!");
                        val_rest -= value;
                        System.out.print(""+pre_value+":");
                        e.theDeltaTime = pre_value;
                    }
                    double_note.add(e.theTone);
                }
                theEventList.add(e);
//                if (theToneCnt < 20 && e.theCommand<= 0x90 ){
//                    System.out.println("note="+Integer.toHexString(e.theTone)+" c="+Integer.toHexString(e.theChanal)+" cmd:"+Integer.toHexString(e.theCommand));
//                }
                value = parseFlexNumber(bdis);
                
//                theToneCnt++;
            }
            bdis.close();
            System.out.println("ges_midi_time="+ges_midi_time);
            
        }
        
        private void parseEvent(MTEvent e, DataInputStream dis, int pre_command) throws Exception{
            int command = dis.read();
            e.theCommand = command;
            if (command <0x80){
                //System.out.println("no command:"+command+" pre="+Integer.toHexString(pre_command));
                int val=dis.read();
                if ((pre_command & 0xF0) == 0x80) {
                    e.theCommand = pre_command & 0xF0;
                    e.theChannel = pre_command & 0x0F;
                    e.theTone = command;
                } else if ((pre_command & 0xF0) == 0x90) {
                    e.theCommand = pre_command & 0xF0;
                    e.theChannel = pre_command & 0x0F;
                    e.theTone = command;
                    e.theStrength = val;
                }
            } else if (command == 0xF0 || command == 0xF7 ){
                int len = parseFlexNumber(dis);
                byte[] buffer = new byte[len];
                dis.read(buffer);
            } else if (command == 0xFF){
                int type = dis.read();
                int len = parseFlexNumber(dis);
                byte[] buffer = new byte[len];
                dis.read(buffer);
            } else if ((command & 0xF0) == 0x80) {
                e.theCommand = command & 0xF0;
                e.theChannel = command & 0x0F;
                e.theTone = dis.read();
                dis.read();       
            } else if ((command & 0xF0) == 0x90) {
                e.theCommand = command & 0xF0;
                e.theChannel = command & 0x0F;
                e.theTone = dis.read();
                e.theStrength = dis.read();
            }else if ((command & 0xF0) == 0xA0 || (command & 0xF0) == 0xB0 || (command & 0xF0) == 0xE0) {
                e.theCommand = command & 0xF0;
                e.theChannel = command & 0x0F;
                dis.read();
                dis.read();       
            }else if ((command & 0xF0) == 0xC0 || (command & 0xF0) == 0xD0 ) {
                e.theCommand = command & 0xF0;
                e.theChannel = command & 0x0F;
                dis.read();       
            } else {
                System.err.println("No Command! "+Integer.toHexString(command));
            }
        }
        
        public int parseFlexNumber (DataInputStream dis) throws IOException{
            int value =0;
            int c;
            
            value = dis.read();
            //if (theToneCnt < 20 ) System.out.println("val1="+value);
            if (value < 0 ) return value;
            if (value >= 0x80)
            {
                c = value;
                value =0;
                while (c >= 0x80){
                    value = value * 128 + (c & 0x7f);
                    c = dis.read();
                    //if (theToneCnt < 20 ) System.out.println("c="+c+" val="+value);
                }
                value = value *128 + (c & 0x7f);
            }
            //if (theToneCnt < 20 ) System.out.println("valout="+value);
            return (value);
        }
    }
    
    public class MTEvent{
        int theDeltaTime;
        
        int theCommand;
        
        int theChannel;
        
        int theTone;
        
        int theStrength;
    }
    
}
