/* (c) by Thomas Oltzen
 * Created on 29.09.2011
 *
 */
package de.oltzen.pianoteacher;

import java.util.logging.Level;
import java.util.logging.Logger;

import ch.ntb.usb.*;
import ch.ntb.usb.logger.LogUtil;

public class USBTest {

    final static int PSCMD_ECHO = 0;

    final static int PSCMD_GET = 1;

    final static int PSCMD_SET = 2;

    final static int PSCMD_NAME = 3;

    final static int PSCMD_ERROR = 4;

    static int up = 1;

    private static void logBus()
    {
        // if you don't use the ch.ntb.usb.Device class you must initialise
        // Libusb before use
        LibusbJava.usb_init();
        LibusbJava.usb_find_busses();
        LibusbJava.usb_find_devices();

        // retrieve a object tree representing the bus with its devices and
        // descriptors
        Usb_Bus bus = LibusbJava.usb_get_busses();

        // log the bus structure to standard out
        Utils.logBus(bus);
    }

    private static void logData(byte[] data)
    {
        System.out.print("Data: ");
        for (int i = 0; i < data.length; i++) {
            System.out.print("0x" + Integer.toHexString(data[i] & 0xff) + " ");
        }
        System.out.println();
    }

    public static void main(String[] args)
    {
        Logger l = LogUtil.getLogger("ch.ntb.usb");
        l.setLevel(Level.ALL);
       
        logBus();
        // get a device instance with vendor id and product id
        boolean retry = true;
        int counter = 0;
        Device dev = null;
        try {
            dev = USB.getDevice((short) 0x16c0, (short) 0x5dc);
            dev.open(1, 0, -1);
            System.out.println("des=" + dev.getConfigDescriptors().length);
            System.out.println("des=" + dev.getConfigDescriptors()[0].getBConfigurationValue());
            System.out.println("des=" + dev.getAltinterface());


            // dev.open(0, 0, -1);
            String text = "PC Computer und wir sehen nun die Lampen leuchten! ";
            text = text + "/* This function is called by the driver to provide a control transfer's";
            text = text + " * payload data (control-out). It is called in chunks of up to 8 bytes. The";
            text = text + " * total count provided in the current control transfer can be obtained from";
            text = text + " * the 'length' property in the setup data. If an error occurred during";
            text = text + " * processing, return 0xff (== -1). The driver will answer the entire transfer";
            text = text + " * with a STALL token in this case. If you have received the entire payload";
            text = text + " * successfully, return 1. If you expect more data, return 0. If you don't";
            text = text + " * know whether the host will send more data (you should know, the total is";
            text = text + " * provided in the usbFunctionSetup() call!), return 1.";
            text = text + " * NOTE: If you return 0xff for STALL, 'usbFunctionWrite()' may still be called";
            text = text + " * for the remaining data. You must continue to return 0xff for STALL in these";
            text = text + " * calls.";
            text = text + " * In order to get usbFunctionWrite() called, define USB_CFG_IMPLEMENT_FN_WRITE";
            text = text + " * to 1 in usbconfig.h and return 0xff in usbFunctionSetup()..";
            System.out.println("n="
                    + Integer.toHexString(USB.REQ_TYPE_TYPE_VENDOR | USB.REQ_TYPE_RECIP_DEVICE
                            | USB.REQ_TYPE_DIR_DEVICE_TO_HOST));
            dev.controlMsg(USB.REQ_TYPE_TYPE_VENDOR | USB.REQ_TYPE_RECIP_DEVICE , 1,
                    2, 256, text.getBytes(), text.length(), 100000, true);

            dev.close();
            retry = false;
            System.out.println("Success=" + counter);
        } catch (USBException e) {
            counter++;

            e.printStackTrace();
        }
    }

}
