//$Id: IntArray.java,v 1.1 2009/09/02 20:08:22 Administrator Exp $
package de.oltzen.util;

import java.io.*;

public class IntArray implements Serializable
{

	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -7893356583255810238L;

    private int theSize;

	private int[] theArray;

	public IntArray ()
	{
		theArray = new int[0];
	}

    public IntArray (int ensure)
    {
        theArray = new int[ensure];
    }

	public IntArray (int[] array)
	{
		theArray = array;
		theSize = array.length;
	}

	/** Ensure that the internal array has a suitable length. The perfomance
	* of the <code>put</code> method can be accelarating.
	*/
	public void ensure (int max_index)
	{
		if (theArray.length <= max_index)
		{
			int[] newarray = new int[max_index+1];
			System.arraycopy(theArray, 0, newarray, 0 , theArray.length);
			theArray = newarray;
		}
	}

	public int get (int index)
	{
		return theArray.length <= index ? 0 : theArray[index];
	}

	public void put (int index, int value)
	{
		if (theArray.length <= index)
		{
		    int new_len = index+1;
		    if (new_len < theArray.length*2/3) new_len = theArray.length*2/3;
			int[] newarray = new int[new_len];
			System.arraycopy(theArray, 0, newarray, 0 , theArray.length);
			theArray = newarray;
		}
		if (index+1 > theSize) theSize= index+1;
		theArray[index] = value;
	}

	public void add (int value)
	{
		put(theSize, value);
	}

	public void setSize(int size)
	{
		if (theSize > size)
		{
			theSize = size;
		}
		else
		{
			put(size, 0);
		}
	}

	public int size()
	{
		return theSize;
	}
    
    /** Return the internal array size. */
    public int getEnsureSize()
    {
        return theArray.length;
    }

	/** Return the position in the array. If result -1, then is the number not
	* in the array.
	*/
	public int searchPosition(int num)
	{
		for (int i= 0; i < theSize ; i ++){
			if (theArray[i]==num) return i;
		}
		return -1;
	}

	public int[] getArray()
	{
		int[] newarray = new int[theSize];
		System.arraycopy(theArray, 0, newarray, 0 , theSize);
		return newarray;
	}
}