/* (C) 2011 Thomas Oltzen 
 * this code is licensed under the GPL */

#include <util/twi.h> //enthlt z.B. die Bezeichnungen fr die Statuscodes in TWSR
#include <avr/interrupt.h> //dient zur behandlung der Interrupts
#include <stdint.h> //definiert den Datentyp uint8_t


// Defines an den Controller und die Anwendung anpassen


#define F_CPU 8000000L                  // Systemtakt in Hz

 
// ab hier nichts ndern, wird alles berechnet
#define T_S_MS_TAKT (F_CPU/1000L) // Takte pro 1ms 
//#define T_PWM 1   // fr TESTs



uint8_t note[16];

uint8_t b_port_cnt = 0;

//uint16_t tcnt =0;

//uint8_t note_cnt=0;

ISR(TIMER0_OVF_vect)
{
	uint8_t notebase;
	PORTB = 255;
	notebase = (b_port_cnt&0xFE)*3;
	notebase = notebase>>1;
	while (note[notebase]==0 && note[notebase+1] ==0 && note[notebase+2]==0 && b_port_cnt!=0){
		b_port_cnt++;
		b_port_cnt &= 7;
		notebase = (b_port_cnt&0xFE)*3;
		notebase = notebase>>1;
	}
	if (b_port_cnt&1){
		PORTC = PORTC & 0xf0 | 0xf & (note[notebase+1]>>4);
		PORTD = note[notebase+2];
	
	} else {
		PORTD = note[notebase+1]<<4 | (note[notebase])>>4;
		PORTC = PORTC & 0xf0 | 0xf & (note[notebase]);
	}
	PORTB = ~(1<<b_port_cnt);
/*
	tcnt++;
	if (tcnt==1000){
		note[note_cnt>>3]=0;
		note_cnt++;
		if (note_cnt>88)note_cnt=0;
		note[note_cnt>>3]=(1<<(note_cnt&7));
		tcnt = 0;

	}
*/
	b_port_cnt++;
	b_port_cnt &= 7;
}

#define TWI_ADR 128


/*Initaliserung des TWI-Interface. Muss zu Beginn aufgerufen werden, sowie bei einem Wechsel der Slave Adresse
Parameter: adr: gewnschte Slave-Adresse
*/
void init_twi_slave ()
{
	TWAR = TWI_ADR; //Adresse setzen
	TWCR &= ~(1<<TWSTA)|(1<<TWSTO);
	TWCR |= (1<<TWEA) | (1<<TWEN)|(1<<TWIE); 
}

//Je nach Statuscode in TWSR mssen verschiedene Bitmuster in TWCR geschreiben werden(siehe Tabellen im Datenblatt!). 
//Makros fr die verwendeten Bitmuster:

//ACK nach empfangenen Daten senden/ ACK nach gesendeten Daten erwarten
#define TWCR_ACK TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);  
//NACK nach empfangenen Daten senden/ NACK nach gesendeten Daten erwarten     
#define TWCR_NACK TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
//switched to the non adressed slave mode...
#define TWCR_RESET TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);  

/*ISR, die bei einem Ereignis auf dem Bus ausgelst wird. Im Register TWSR befindet sich dann 
ein Statuscode, anhand dessen die Situation festgestellt werden kann.
*/
ISR (TWI_vect)  
{
	uint8_t data=0;

	switch (TW_STATUS) //TWI-Statusregister prfen und ntige Aktion bestimmen 
	{

		case TW_SR_SLA_ACK: // 0x60 Slave Receiver, wurde adressiert	
			TWCR_ACK; // nchstes Datenbyte empfangen, ACK danach
			break;

		case TW_SR_DATA_ACK: // 0x80 Slave Receiver,Daten empfangen
			data=TWDR; //Empfangene Daten auslesen
			TWCR_ACK;
			if (data==255){
				for (int i = 0;i < 16 ; i++) note[i]=0;
			} else if (data>=128){
				data = (data&0x7F)-21;
				note[(data>>3) & 0xf] |= (1 << (data & 7)); 
			} else {
				data = data - 21;
				note[(data>>3) & 0xf] &= ~(1 << (data & 7)); 
			}
			break;

		case TW_ST_SLA_ACK: //?!?
		case TW_ST_DATA_ACK: //0xB8 Slave Transmitter, weitere Daten wurden angefordert
		case TW_ST_DATA_NACK: //0xC0 Keine Daten mehr gefordert 
		case TW_SR_DATA_NACK: //0x88 
		case TW_ST_LAST_DATA: //0xC8  Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received
		case TW_SR_STOP: // 0xA0 STOP empfangen
		default: 	
			TWCR_RESET; //bertragung beenden, warten bis zur nchsten Adressierung
			break;


	} //end.switch (TW_STATUS)
	PORTB &= ~0x08;
} //end.ISR(TWI_vect)
 


int main(void) {

	DDRB = 0xFF; // All output pins
	PORTB = 0x00;
	DDRD = 0xFF;
	PORTD = 0x00;
	DDRC = 0xFF; 
	PORTC = 0x00; 

	init_twi_slave();

	TIMSK0 |= (1<<TOIE0) ; 
	TCCR0B = (1<<CS01); //Prescale=8,

	for (int i = 0;i < 16 ; i++){
		note[i]=0;
	}

	sei();

	while(1){
	
//		PORTB != 0xfC; 
//		PORTB &= 0x02; 
		}
}
