
#include "lcd.h"
#include "twimaster.h"
#include "midi.h"
#include <avr/io.h>
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>
#include "usbdrv/usbdrv.h"

#define I2C_EE_ADR 0xA0
#define I2C_BAR_ADR 0x80
#define I2C_SYN_ADR 0x82


uint16_t ee_read_cnt;
uint16_t ee_read_len;

uint16_t ee_write_adr;
uint16_t ee_read_adr;
uint16_t ee_buffer_cnt;
uint8_t ee_buffer[128];

uint16_t ee_stored EEMEM=0;

char show_ee_flag=0;

uint16_t show_adr = 0;

uint16_t led=0;
uint16_t keyboard=0;

uint8_t no_key_cnt = 0;

uint32_t clock_ms = 0; // count the ms;
uint16_t clock_us = 0; // clock in mikro second

uint8_t note_send_cnt=0;
uint8_t note_buffer[128];

/* global var for midi stream */

uint8_t start_flag=0;

uint16_t delay_unit_ms;

uint32_t playtime_ms = 0;

uint32_t showtime_ms=0;

uint32_t starttime_ms=0;

uint32_t looptime_ms=0;

uint8_t select_mode;

uint8_t mode_cursor;

uint8_t time_input[5];
static const uint8_t time_input_pos[] = {0, 1, 3, 4, 6};


uint16_t chanal =0;

#define MCMD_END 0x3B
#define MCMD_CHANAL 0x3E
#define MCMD_PAUSE 0x3D
#define MCMD_DELAY_UNIT 0x3F
#define MCMD_DELAY_1BYTE 0x31
#define MCMD_DELAY_2BYTES 0x32
#define MCMD_DELAY_4BYTES 0x34
/* write the external eeprom data to the LCD */

/*
void show_ee_data(){
	uint8_t c;
	uint16_t len;
	len = eeprom_read_word(&ee_stored);
	if (len ==0) return;
	if(show_adr+15>len){
		show_adr =0 ;
	}
	lcd_gotoxy(0,1);
	lcd_hex16(len);
	lcd_gotoxy(0,0);
	i2c_start_wait(I2C_EE_ADR+I2C_WRITE);     // set device address and write mode
    i2c_write(show_adr >> 8);                        // write address = 5
    i2c_write(show_adr & 0xFF );                        // write value 0x75 to EEPROM
	i2c_rep_start(I2C_EE_ADR+I2C_READ);
	for (int i=0 ; i < 16 && show_adr+i<len;i++){ //(&& show_adr+i<len)

		c = i2c_readAck();                    // read one byte from EEPROM
		lcd_putc(c);
	}
	i2c_readNak();
    i2c_stop();
	show_adr++;
}
*/

/* write the buffer to external eeprom */

void ee_write_buffer()
{
	if ( ee_buffer_cnt==0) return;
	i2c_start_wait(I2C_EE_ADR+I2C_WRITE);     // set device address and write mode
    i2c_write(ee_write_adr >> 8);                        // write address = 5
    i2c_write(ee_write_adr & 0xFF );                        // write value 0x75 to EEPROM

	for (int i = 0; i < ee_buffer_cnt ; i++){
		i2c_write(ee_buffer[i]);
	}
    i2c_stop();
	ee_write_adr += ee_buffer_cnt;
	ee_buffer_cnt =0;
}

void ee_read_buffer()
{
	ee_buffer_cnt = 0;
	i2c_start_wait(I2C_EE_ADR+I2C_WRITE);     // set device address and write mode
    i2c_write(ee_read_adr >> 8);                        // write address = 5
    i2c_write(ee_read_adr & 0xFF );                        // write value 0x75 to EEPROM
	i2c_rep_start(I2C_EE_ADR+I2C_READ);
	for (int i=0 ; i < 128;i++){ //(&& show_adr+i<len)
		ee_buffer[i] = i2c_readAck();                    // read one byte from EEPROM
	}
	i2c_readNak();
    i2c_stop();

}

uint8_t get_ee_byte()
{
	uint16_t len;
	uint8_t out;
	len = eeprom_read_word(&ee_stored);
//	if (ee_read_adr >= len) return  MCMD_END;
	if (ee_read_adr==0 || ee_buffer_cnt>=128){
		ee_read_buffer();
	}
	out = ee_buffer[ee_buffer_cnt];
	ee_read_adr++;
	ee_buffer_cnt++;
	return out;
}

uint16_t get_ee_uint16()
{
	return ((uint16_t) get_ee_byte()) << 8 | get_ee_byte();
}

uint32_t get_ee_uint32()
{
	return get_ee_byte() << 24 | get_ee_byte() << 16 | get_ee_byte() << 8 | get_ee_byte();
}
/* ------------------------------------------------------------------------- */

uint16_t readADChannel(uint8_t mux)
{
  uint8_t i;
  uint16_t result;

  ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) ;    // Frequenzvorteiler
                               // setzen auf 64 (1) und ADC aktivieren (1)

  ADMUX = mux;                      // Kanal waehlen
  ADMUX |= (1<<REFS0); // AVCC Referenzspannung nutzen

  /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
     also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
  ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung
  while ( ADCSRA & (1<<ADSC) ) {
     ;     // auf Abschluss der Konvertierung warten
  }
  result = ADCW;  // ADCW muss einmal gelesen werden,
                  // sonst wird Ergebnis der nchsten Wandlung
                  // nicht bernommen.

  /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
  result = 0;
  for( i=0; i<4; i++ )
  {
    ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
    while ( ADCSRA & (1<<ADSC) ) {
      ;   // auf Abschluss der Konvertierung warten
    }
    result += ADCW;		    // Wandlungsergebnisse aufaddieren
  }
  ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)

  result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert

  return result;
}
/* ------------------------------------------------------------------------- */

uchar usbFunctionWrite(uchar *data, uchar len)
{
	for (int i = 0; i < len ; i++){
		//lcd_putc(data[i]);
		ee_buffer[ee_buffer_cnt]= data[i];
		ee_buffer_cnt++;
		if (ee_buffer_cnt==128){
			ee_write_buffer();
		}
	}
	ee_read_cnt += len;
	if (ee_read_cnt == ee_read_len) {
		ee_write_buffer();
		show_ee_flag=1;
		eeprom_write_word(&ee_stored, ee_read_len);
		lcd_gotoxy(0,1);
	    lcd_puts("Music Data Saved");
		return 0;
	}
	return 1;
}


uchar	usbFunctionSetup(uchar data[8]){
 	usbRequest_t *rq = (void *)data;   // cast to structured data for parsing

	lcd_gotoxy(0,1);
	lcd_puts("Load Music Data ");

    switch(rq->bRequest){
    case 1:
		lcd_gotoxy(0,0);
		ee_read_cnt = 0;
		ee_write_adr = rq->wIndex.word;
		ee_buffer_cnt = 0;
		ee_read_len = rq->wLength.word;


	    return USB_NO_MSG;        // tell driver to use usbFunctionWrite()
    }
    return 0;                               // ignore all unknown requests
}



/* ------------------------------------------------------------------------- */
/* ------------------- Key & LED performer  -------------------------------- */


uint8_t key_led_row;

/* Read the keys and put the pressed key into the variable keyboard. */
void perform_key(){
	uint8_t k;
	DDRA = DDRA & 0xf; // Key Input PA4-PA7
	PORTA = PORTA | 0xF0; // Key Input pull up PA4-PA7
	PORTD |= 0xF0; // Ensure that all LEDs are off.
	//PORTC &= 0xf;
	key_led_row &=3;
	PORTC = ((~(1<<key_led_row))<<4) | (PORTC & 0xf) | 0xb; // (~(1<<key_led_row))<<4 ;
    _delay_ms(1);
	k=((~PINA)>>4);
	k&=0xf;
	if (k>0){
		keyboard = k  << (key_led_row <<2);
		no_key_cnt =0;
	} else {
		no_key_cnt++;
	}
}

/* Set the led for a short time on. */
void perform_led(){
	uint8_t out;
	DDRA = DDRA | 0xF0; // Output PA4-PA7
	PORTD |= 0xF0; // Ensure that all LEDs are off.

	key_led_row &=3;
	out = (led >> ( (key_led_row <<2))) & 0xf;
	PORTA = (out<<4) | (PORTA & 0xf); // (~(1<<key_led_row))<<4 ;
	PORTD = (~(1<<key_led_row) <<4) | (PORTD & 0xf);
	if (out !=0)
	 	_delay_ms(1);
}


/* ------------------------------------------------------------------------- */


ISR(TIMER2_OVF_vect) /* veraltet: SIGNAL(SIG_OVERFLOW0) */
{
//	TCNT1 = 0xFF00;
    /* Interrupt Code */
	if (!start_flag) return;
	clock_us+=255;
	if (clock_us>12000){
		clock_us-=12000;
		clock_ms++;
	}
}


/* ------------------------------------------------------------------------- */

void lcd_time(uint32_t time_ms){
	// M:SS.z z:= hundredth second.

	lcd_putc('0'+(time_ms/600000));
	lcd_putc('0'+((time_ms/60000)%10));
	lcd_putc(':');
	time_ms = time_ms % 60000;
	if (time_ms < 10000) {
		lcd_putc('0');
	} else {
		lcd_putc('0'+(time_ms/10000));
		time_ms = time_ms % 10000;
	}
	lcd_putc('0'+(time_ms/1000));
	lcd_putc('.');
	lcd_putc('0'+((time_ms%1000)/100));
}

void ignore_notes(){
	uint8_t note;
	note =1;
	while(note!=0){
		note = get_ee_byte();
	}
}

void print_seleced_mode()
{
	lcd_gotoxy(0,1);
	lcd_puts("                ");
	lcd_gotoxy(0,0);
	lcd_puts("Mode: ");
	if (select_mode==0){
		lcd_puts("Sel.Chanal");
	} else if (select_mode==1){
		lcd_puts("Start Time");
		lcd_gotoxy(0,1);
		lcd_time(starttime_ms);
		lcd_gotoxy(0,1);
	} else if (select_mode==2){
		lcd_puts("Loop Time ");
		lcd_gotoxy(0,1);
		lcd_time(looptime_ms);
		lcd_gotoxy(0,1);
	} 
}

uint8_t get_keynum()
{
	for (int i=0; i < 10;i++){
		if (keyboard & (1 << i)) return ((i+1))%10;
	}
	return 0;
}

void time_input_set_time(uint32_t time)
{
	time_input[0] = time/600000UL;
	time_input[1] = (time/60000UL)%10;
	time = (time % 60000UL)/100;
	time_input[2] = time/100;
	time_input[3] = (time/10)%10;
	time_input[4] = time%10;
}

uint32_t time_input_get_time()
{
	return time_input[0]*600000UL+time_input[1]*60000UL+time_input[2]*10000UL+time_input[3]*1000UL+time_input[4]*100UL;
}

/** main routine for select the chanal */
void wait_for_start()
{
	uint8_t no_mode_key = 1;
	start_flag= 0;
	TIMSK &= ~(1<<TOIE2);		// Timer dekativieren wegen USB
	print_seleced_mode();
	// Midi device silence
	midi_all_off();
	while((PINA&2)){
		key_led_row++;
        usbPoll();
		perform_key();
		perform_led();
		//PORTC |= 0x0B ;
		//lcd_gotoxy(0,1);
		//lcd_hex8(PINC);
		//lcd_hex8(PORTC);
		if ((PINC&4)==4) {
			no_mode_key=1;
		} else if (no_mode_key==1 && (PINC&4)==0){
			no_mode_key=0;
			mode_cursor=0;
			select_mode ++;
			if (select_mode>=3) select_mode=0;
			print_seleced_mode();
			lcd_gotoxy(0,1);
		}
		if ( no_key_cnt > 10 && keyboard){
			if (select_mode==0){
				chanal = ~chanal & keyboard | chanal & ~keyboard;
			} else if (select_mode==1){
				time_input_set_time(starttime_ms);
				time_input[mode_cursor]=get_keynum();
				starttime_ms=time_input_get_time();
				lcd_gotoxy(0,1);
				lcd_time(starttime_ms);
				mode_cursor++;
				if (mode_cursor>4) mode_cursor=0;
				//lcd_gotoxy(8,1);
				//lcd_hex8(mode_cursor);
				lcd_gotoxy(time_input_pos[mode_cursor],1);
			} else if (select_mode==2){
				time_input_set_time(looptime_ms);
				time_input[mode_cursor]=get_keynum();
				looptime_ms=time_input_get_time();
				lcd_gotoxy(0,1);
				lcd_time(looptime_ms);
				mode_cursor++;
				if (mode_cursor>4) mode_cursor=0;
				lcd_gotoxy(time_input_pos[mode_cursor],1);
			}
			keyboard =0 ;
		}
		led = chanal;
	}
	lcd_gotoxy(0,1);
	lcd_puts("2. Sec");
	_delay_ms(1000);
	lcd_gotoxy(0,1);
	lcd_puts("1. Sec");
	_delay_ms(1000);
	start_flag= 1;
}

void send_note_buffer()
{
	//lcd_hex8(note_send_cnt);
	i2c_start_wait(I2C_BAR_ADR+I2C_WRITE);     // set device address and write mode
	for (int i=0; i < note_send_cnt ; i++)
    	i2c_write(note_buffer[i]);
	i2c_stop();
	i2c_start_wait(I2C_SYN_ADR+I2C_WRITE);     // set device address and write mode
	for (int i=0; i < note_send_cnt ; i++)
    	i2c_write(note_buffer[i]);
	i2c_stop();
	
	for (int i=0; i < note_send_cnt ; i++)
    	midi_play_tone(note_buffer[i]);

	note_send_cnt = 0;	
}

void unlight_notes()
{
	//lcd_hex8(note_send_cnt);
	i2c_start_wait(I2C_BAR_ADR+I2C_WRITE);     // set device address and write mode
	i2c_write(255);
//	for (int i=21; i < 21+88 ; i++)
  //  	i2c_write(i);
	i2c_stop();
	i2c_start_wait(I2C_SYN_ADR+I2C_WRITE);     // set device address and write mode
	i2c_write(255);
	i2c_stop();

	
	note_send_cnt = 0;	
	
}


void read_notes(uint8_t command, uint8_t mask){
	uint16_t ch;
	uint8_t note;
	ch = command & 0xf;
	if ((1<<ch)&chanal){
		note =1;
		while(note!=0){
			note = get_ee_byte();
			if (note != 0){
				note_buffer[note_send_cnt]=note|mask;
				note_send_cnt++;
				note_send_cnt&=0x7F; // Ensure a buffer overflow
			}
		}
	} else {
		ignore_notes();
	}
}

void parse_chanal()
{
	uint16_t eelen;
	uint8_t command; 
	eelen = eeprom_read_word(&ee_stored);
//	if (eelen < 0xFFFF){
		ee_read_adr = 0;
		command = get_ee_byte();
//		lcd_gotoxy(0,0);
//		lcd_hex8(command);
		if (command==MCMD_CHANAL) {
			chanal = ((uint16_t) get_ee_byte()) << 8 | get_ee_byte();
			led = chanal;
//			lcd_gotoxy(4,0);
///			lcd_hex16(chanal);
		}
		ee_read_adr = 0;
//	} else {
//		lcd_gotoxy(6,0);
//			lcd_hex16(eelen);
//	}
}

// Precondition : command==MCMD_DELAY_1BYTE || command==MCMD_DELAY_2BYTES || command==MCMD_DELAY_4BYTES
uint32_t get_delaytime(uint8_t command)
{
	if (command==MCMD_DELAY_1BYTE){
		return get_ee_byte();
	} else if (command==MCMD_DELAY_2BYTES){
		return get_ee_uint16();
	} 
	return get_ee_uint32();
				
}

void move_to_ms(uint32_t totime)
{
	uint32_t delay_time;
	playtime_ms=0;
	ee_read_adr=0;
	while(playtime_ms < totime){

		uint8_t command = get_ee_byte();
		if (command==MCMD_END){
			ee_read_adr=0;
			playtime_ms = 0;
			clock_ms =0;
			showtime_ms =0;
			return;
		} else if (command==MCMD_CHANAL) {
			get_ee_uint16();
		} else if (command==MCMD_PAUSE) {

		} else if (command==MCMD_DELAY_1BYTE || command==MCMD_DELAY_2BYTES || command==MCMD_DELAY_4BYTES){
			delay_time = get_delaytime(command);
			playtime_ms +=delay_time;
		} else if ((command & 0xf0)==0x10) { // Notes off command
			read_notes(command, 0);
			note_send_cnt =0;
		} else if ((command & 0xf0)==0x20) { // Notes on command
			read_notes(command, 0x80);
			note_send_cnt =0;
		} else if ((command & 0xf0)==0x40) { // Display a text into LCD
			uint8_t note =1;
			while(note!=0){
				note = get_ee_byte();
			}
		}

	}
	showtime_ms = playtime_ms;
	clock_ms = showtime_ms;
	unlight_notes();
}

void play()
{
	uint8_t command ;
	uint8_t note;
	uint16_t ch;
	uint16_t eelen;
	uint32_t next_outtime=0;
	uint32_t delay_time;
	uint16_t speed = 0;
	uint8_t speed_protz;
	uint8_t wait_till_unkey=0;
	uint8_t first_time=1;
	//uint32_t loop_from_ms =0;
	//uint32_t loop_to_ms =0;
	//playtime_ms=0;
		// Timer
	if (PINC & (1<<3)){
		lcd_gotoxy(0,1);
		lcd_puts("No LED Bar!     ");
		_delay_ms(2000);
		return;
	}
	// Midi device silence
	midi_all_off();
	move_to_ms(showtime_ms);
	speed = readADChannel(0);
	eelen = eeprom_read_word(&ee_stored);
	lcd_clrscr();
	unlight_notes();

	TIMSK |= (1<<TOIE2);		// Interrupts aktivieren und damit Timer starten
	while((PINA&2)){
		key_led_row++;
        usbPoll();
		perform_key();
	
		if (playtime_ms>clock_ms){
			if (wait_till_unkey){
				lcd_gotoxy(12,0);
				lcd_hex16(keyboard);
				if (no_key_cnt>20){
					wait_till_unkey=0;
					keyboard=0;
				}
			} else {
				lcd_gotoxy(12,0);
				lcd_hex16(keyboard);
				if (keyboard & 0x8000){
					if (showtime_ms >10000) {
						showtime_ms-=10000;
					} else {
						showtime_ms=0;
					}
					move_to_ms(showtime_ms);
					next_outtime=clock_ms-1;
					wait_till_unkey = 1;
				}
				if (keyboard & 0x4000){
					if (starttime_ms>0 && looptime_ms>0){
						starttime_ms =0;
						looptime_ms =0;
					} else if (starttime_ms==0){
						starttime_ms = showtime_ms;
					} else if (looptime_ms==0){
						looptime_ms = showtime_ms;
						move_to_ms(showtime_ms);
					}
					next_outtime=clock_ms-1;
					wait_till_unkey = 1;
				}
			}
			if (looptime_ms>0 && looptime_ms<showtime_ms){
				move_to_ms(starttime_ms);
				next_outtime=clock_ms-1;
			}
			if (next_outtime<clock_ms){
				lcd_gotoxy(0,1);
				lcd_puts("T:");
				lcd_time(clock_ms);
				lcd_putc(' ');
				//lcd_hex16(playtime_ms>>16);
				//lcd_hex16(playtime_ms&0xFFFF);
				next_outtime = clock_ms + 100;
				speed = readADChannel(0);
				speed_protz = speed*100UL/1023;
				if (speed_protz<=99){
					lcd_putc('0'+(speed_protz/10));
					lcd_putc('0'+(speed_protz%10));
					lcd_puts("% ");
				} else {
					lcd_puts("100%");
				}
				lcd_gotoxy(0,0);
				lcd_puts("R:");
				lcd_time(showtime_ms);
				lcd_putc(' ');				
			}
		} else {
		//	if(ee_read_adr>=eelen){
		//		lcd_gotoxy(0,0);
		//		lcd_puts("Finish ");
		//		lcd_hex16(ee_read_adr);
		//		lcd_hex16(eelen);
		//		return;
		//	}
			command = get_ee_byte();
			//lcd_gotoxy(8,0);
			//lcd_hex8(command);
			if (command==MCMD_END){
				ee_read_adr=0;
				playtime_ms = 0;
				clock_ms =0;
				showtime_ms = 0;
				return;
			} else if (command==MCMD_CHANAL) {
				ch =  get_ee_uint16();
				if (chanal==0){
					chanal = ch;
					return;
				}
			} else if (command==MCMD_PAUSE) {
				return;

			} else if (command==MCMD_DELAY_1BYTE || command==MCMD_DELAY_2BYTES || command==MCMD_DELAY_4BYTES){
				delay_time = get_delaytime(command);
				showtime_ms +=delay_time;
				if (!first_time){	
					if (speed<11){
						playtime_ms+=1000;
					} else {
						playtime_ms+=delay_time*1023/speed;
					}
				}
				first_time=0;
			} else if ((command & 0xf0)==0x10) { // Notes off command
				read_notes(command, 0);
				//lcd_gotoxy(0,0);
				//lcd_putc('R');
				//lcd_hex8(note_send_cnt);
			} else if ((command & 0xf0)==0x20) { // Notes on command
				read_notes(command, 0x80);
				//lcd_gotoxy(0,0);
				//lcd_putc('S');
				//lcd_hex8(note_send_cnt);
			} else if ((command & 0xf0)==0x40) { // Display a text into LCD
				ch = command & 0xf;
				lcd_gotoxy(0, ch);
				note =1;
				while(note!=0){
					note = get_ee_byte();
					if (note!=0){
						lcd_putc(note);
					}
				}

			}
			if (note_send_cnt){
				send_note_buffer();
			}

		}
	}
}


/* ------------------------------------------------------------------------- */
int main(void)
{
	//uint16_t sec=0;
	//DDRD = ~USBMASK;
	// PD0 : UP Key
	// PD1 : DOWN Key 
	// PD4-PD7 : LEDs Output
	DDRD = 0xF0 ; 
	PORTD = 3;     /* USB-Verbindung */ // Pullout for Up and Down key
	//_delay_ms(1000);
	DDRA &= ~2; // PA1 is the Start Stop button
	PORTA |= 2;//PA1 has Pullput

	DDRC = 0xF0 ; // LED & Key Row select PC4-PC7 PC3:LED BAR Connect PC2: Mode Key
	PORTC = 0x0B ; //PC3, PC4 PULL Up

	// PB4 : UP Key
	// PB5 : DOWN Key 
	DDRB &= 0x0F;


	PORTB &= ~(0x02);

	i2c_init();
	midi_init();
	midi_all_off();
	lcd_init();
	lcd_clrscr();
	lcd_home();
	lcd_set_cursor_control(1, 1);// Blink & show cursor
	lcd_puts("Piano Trainer");
	lcd_gotoxy(0,1);
	lcd_puts("(c) T. Oltzen");

    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */

    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(1000);
	eeprom_busy_wait();
	usbDeviceConnect();
    usbInit();

	// Timer
	TCCR2 |= (1<<CS20);

	parse_chanal();

    sei();
	for(;;){      /* main event loop */
		wait_for_start();
		_delay_ms(1);
	 	while((PINA&2)==0)usbPoll();
		_delay_ms(1);
		
		play();	
		//	 	while((PINA&2)){
//			if ((clock_ms/1000)>sec){
//				lcd_gotoxy(0,1);
//				sec = clock_ms/1000;
//				lcd_hex16(sec);
//			}
//			usbPoll();
//		}
		_delay_ms(1);
		while((PINA&2)==0)usbPoll();
		_delay_ms(1);
	 }
    return 0;
}
