
#include "midi.h"
#include <avr/io.h>

// FOR MIDI
#define UART_BAUD_RATE 31250


//----------------------------------------------------------
// UART

// Send UART byte
//void midi_send(unsigned char byte) {
//	loop_until_bit_is_set(UCSRA, UDRE);//while (!(UCSRA & (1<<UDRE)));
//	UDR = byte;
//}


// Sends to all channels
void midi_all_ch(unsigned char data) {
    unsigned char i;
    for (i=0; i<16; i++) {
        midi_send(0xB0+i);
        midi_send(data);
        midi_send(0x00);
    }
}


// All midi off
void midi_all_off(void) {
    midi_all_ch(0x7B);
    midi_all_ch(0x79);
}

void midi_init()
{
	// UART
	// Hilfsmakro zur UBRR-Berechnung ("Formel" laut Datenblatt)
	#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)
	//UBRRH = (uint8_t)(UART_UBRR_CALC(UART_BAUD_RATE,F_CPU)>>8);
	//UBRRL = (uint8_t)UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);
	//UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);//|(1<<UDRIE);




    //uint16_t ubrr = (uint16_t) ((uint32_t) F_CPU/(16*UART_BAUD_RATE) - 1);
	uint16_t ubrr =  UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);

    UBRRH = (uint8_t) (ubrr>>8);
    UBRRL = (uint8_t) (ubrr);
 
    // UART Receiver und Transmitter anschalten 
    // Data mode 8N1, asynchron 
    UCSRB = (1 << TXEN);
    //UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);

    // Flush Receive-Buffer (entfernen evtl. vorhandener ungltiger Werte) 
    do
    {
        UDR;
    }
    while (UCSRA & (1 << RXC));
}


void midi_send (const uint8_t c)
{
    // Warten, bis UDR bereit ist fr einen neuen Wert 
    while (!(UCSRA & (1 << UDRE)))
        ;

    // UDR Schreiben startet die bertragung 
    UDR = c;
}


void midi_play_tone(uint8_t data){
	if (data==255){
		midi_all_off();
	} else if (data>=128){
		data = (data&0x7F);
		midi_send(0x90);
       	midi_send(data);
	   //midi_send(60);
		midi_send(100);
	} else {
		midi_send(0x80);
        midi_send(data);
		 //midi_send(60);
		 //midi_send(100);
		midi_send(0);
	}
}
